import sys
import unittest
import doctest
import nglview
from nglview import widget, show

import warnings
warnings.filterwarnings('ignore')

PY3 = sys.version_info[0] == 3


try:
    from ipywidgets import Widget
    from ipykernel.comm import Comm
    import nglview
    has_nglview = True

    #------------------------------------------------------
    # Utility stuff from ipywidgets tests: create DummyComm
    # we dont need Jupyter notebook for testing
    #------------------------------------------------------
    class DummyComm(Comm):
        comm_id = 'a-b-c-d'
    
        def open(self, *args, **kwargs):
            pass
    
        def send(self, *args, **kwargs):
            pass
    
        def close(self, *args, **kwargs):
            pass
    
    _widget_attrs = {}
    displayed = []
    undefined = object()

    _widget_attrs['_comm_default'] = getattr(Widget, '_comm_default', undefined)
    Widget._comm_default = lambda self: DummyComm()
    _widget_attrs['_ipython_display_'] = Widget._ipython_display_
    def raise_not_implemented(*args, **kwargs):
        raise NotImplementedError()
    Widget._ipython_display_ = raise_not_implemented
except ImportError:
    has_nglview = False
    nglview = Comm = DummyComm = _widget_attrs = displayed = undefined = Widget  = None

def get_total_errors(modules):
    return sum([doctest.testmod(mod).failed for mod in modules])

@unittest.skipUnless(PY3, 'doctest with py3 only')
def test_nglview_show_module():
    """
    """
    assert not get_total_errors([show,])



import os

def get_fn(fn):
    this_path = os.path.dirname(os.path.abspath(__file__))
    return os.path.join(this_path, 'data', fn)

repr_dict = {'c0': {'0': {'name': 'cartoon',
   'parameters': {'aspectRatio': 5,
    'assembly': 'default',
    'capped': True,
    'clipNear': 0,
    'colorDomain': '',
    'colorMode': 'hcl',
    'colorScale': 'RdYlBu',
    'colorScheme': 'atomindex',
    'colorValue': 9474192,
    'defaultAssembly': '',
    'diffuse': 16777215,
    'flatShaded': False,
    'lazy': False,
    'linewidth': 2,
    'metalness': 0,
    'opacity': 1,
    'radialSegments': 10,
    'radius': 'sstruc',
    'roughness': 0.4,
    'scale': 0.7,
    'sele': 'polymer',
    'side': 'double',
    'smoothSheet': False,
    'subdiv': 6,
    'tension': None,
    'visible': True,
    'wireframe': False}},
  '1': {'name': 'ball+stick',
   'parameters': {'aspectRatio': 2,
    'assembly': 'default',
    'bondSpacing': 0.85,
    'clipNear': 0,
    'colorDomain': '',
    'colorMode': 'hcl',
    'colorScale': '',
    'colorScheme': 'element',
    'colorValue': 9474192,
    'cylinderOnly': False,
    'defaultAssembly': '',
    'diffuse': 16777215,
    'disableImpostor': False,
    'flatShaded': False,
    'lazy': False,
    'lineOnly': False,
    'linewidth': 2,
    'metalness': 0,
    'multipleBond': False,
    'opacity': 1,
    'openEnded': True,
    'radialSegments': 10,
    'radius': 0.15,
    'roughness': 0.4,
    'scale': 1,
    'sele': 'hetero OR mol',
    'side': 'double',
    'sphereDetail': 1,
    'visible': True,
    'wireframe': False}},
  '2': {'name': 'ball+stick',
   'parameters': {'aspectRatio': 2,
    'assembly': 'default',
    'bondSpacing': 0.85,
    'clipNear': 0,
    'colorDomain': '',
    'colorMode': 'hcl',
    'colorScale': '',
    'colorScheme': 'element',
    'colorValue': 9474192,
    'cylinderOnly': False,
    'defaultAssembly': '',
    'diffuse': 16777215,
    'disableImpostor': False,
    'flatShaded': False,
    'lazy': False,
    'lineOnly': False,
    'linewidth': 2,
    'metalness': 0,
    'multipleBond': False,
    'opacity': 1,
    'openEnded': True,
    'radialSegments': 10,
    'radius': 0.15,
    'roughness': 0.4,
    'scale': 1,
    'sele': 'not protein and not nucleic',
    'side': 'double',
    'sphereDetail': 1,
    'visible': True,
    'wireframe': False}}}}



REPRESENTATION_NAME_PAIRS = [
        ('axes', 'axes'),
        ('point', 'point'),
        ('line', 'line'),
        ('rope', 'rope'),
        ('tube', 'tube'),
        ('trace', 'trace'),
        ('label', 'label'),
        ('unitcell', 'unitcell'),
        ('cartoon', 'cartoon'),
        ('licorice', 'licorice'),
        ('distance', 'distance'),
        ('ribbon', 'ribbon'),
        ('surface', 'surface'),
        ('backbone', 'backbone'),
        ('contact', 'contact'),
        ('hyperball', 'hyperball'),
        ('rocket', 'rocket'),
        ('helixorient', 'helixorient'),
        ('simplified_base', 'base'),
        ('spacefill', 'spacefill'),
        ('ball_and_stick', 'ball+stick'),
]

REPRESENTATION_NAMES = list(sorted(set([name for pairs in REPRESENTATION_NAME_PAIRS for name in pairs])))



from notebook.notebookapp import NotebookApp, random_ports
import socket
from tornado import httpserver
from itertools import chain


class NGLViewApp(NotebookApp):
    def get_port(self, port=None):

        # user-given port
        uport = [port,] if port is not None else [self.port,]

        def func():
            return

        self.http_server = httpserver.HTTPServer(func)
        success = None

        for port in chain(uport, random_ports(self.port, self.port_retries+1)):
            try:
                self.http_server.listen(port, self.ip)
            except (OSError, socket.error): 
                pass
            else:
                self.port = port
                success = True
                break
        if not success:
            self.log.critical('ERROR: the notebook server could not be started because '
                              'no available port could be found.')
            self.exit(1)
        self.http_server.stop()
        return self.port



#!/usr/bin/env python

from __future__ import absolute_import
import os, sys, argparse, json
import subprocess
from subprocess import CalledProcessError
from .cmd_example import CMD_EXAMPLE 

bin_path = os.path.join(sys.prefix, 'bin')

simple_source = """
import nglview as nv
""".strip()

demo_source = """
import nglview as nv

view = nv.demo(gui=True)
view
""".strip()

density_source = """
import nglview as nv

view = nv.NGLWidget(gui=True)
view.add_component('filename')
view
""".strip()

def _is_density_data(filename):
    from nglview.utils import FileManager

    try:
        fm = FileManager(filename)

        return fm.ext.lower() in ['dx', 'ccp4', 'mrc', 'map', 'dxbin', 'cube']
    except ValueError:
        return False

notebook_dict = {
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 'null',
   "metadata": {
    "collapsed": True
   },
   "outputs": [],
   "source": [
    "import nglview as nv\n",
    "import pytraj as pt\n",
    "\n",
    "traj = pt.iterload('test.nc', top='prmtop')\n",
    "view = nv.show_pytraj(traj)\n",
    "view"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython",
   "version": "3.5.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}

def install_nbextension(jupyter, user=True):
    path = os.path.dirname(__file__)
    nglview_main = os.path.join(path, 'nglview_main.js')

    local = '--user' if user else ''
    cm_install = '{jupyter} nbextension install {nglview_main} {local}'.format(jupyter=jupyter,
            nglview_main=nglview_main,
            local=local)
    cm_activate = '{jupyter} nbextension enable nglview_main'.format(jupyter=jupyter) 

    with open(os.devnull, 'wb') as devnull:
        subprocess.check_call(cm_install.split(),
            stdout=devnull,
            stderr=subprocess.STDOUT)
        subprocess.check_call(cm_activate.split(),
            stdout=devnull,
            stderr=subprocess.STDOUT)

def disable_extension(jupyter):
    print("disable nglview_main extension")
    cm = '{jupyter} nbextension disable nglview_main'.format(jupyter=jupyter)
    subprocess.check_call(cm.split())


remote_msg = """
Try to use port = {port}

\033[32m In your local machine, run: \033[0m

    {client_cm}

\033[32m NOTE: you might want to replace {hostname} by full hostname with domain name \033[0m

\033[32m Then open your web browser, copy and paste: \033[0m
    http://localhost:{port}/notebooks/{notebook_name}
"""

def get_remote_port(port=None, notebook_path=''):
    import os, socket
    from nglview.scripts.app import NGLViewApp
    port = NGLViewApp().get_port(port=port)

    username = os.getlogin()
    hostname = socket.gethostname()
    client_cm = "ssh -NL localhost:{port}:localhost:{port} {username}@{hostname}".format(username=username,
            hostname=hostname,
            port=port)
    base_notebook_name = os.path.basename(notebook_path)
    print(remote_msg.format(client_cm=client_cm, port=port,
                            hostname=hostname,
                            notebook_name=base_notebook_name))
    return port

def main(notebook_dict=notebook_dict, cmd_arg=sys.argv[1:]):
    PY3 = sys.version_info[0] == 3
    pyv_full_string = ','.join(str(i) for i in sys.version_info)
    pyv_short_string = str(sys.version_info[0])
    default_jexe = ' '.join((sys.executable, '-m jupyter_core'))

    parser = argparse.ArgumentParser(description='NGLView: An IPython/Jupyter widget to '
                                     'interactively view molecular structures and trajectories.',
                                     formatter_class=argparse.RawDescriptionHelpFormatter,
                                     epilog=CMD_EXAMPLE)
    parser.add_argument('command', nargs='?',
            help='command could be a topology filename (.pdb, .mol2, .parm7, ...) or \n'
                          'could be a python script (.py), a notebook (.ipynb). '
                          'If not given, a notebook will be created with only nglview imported') 
    parser.add_argument('traj', nargs='?', help='coordinate filename, optional')
    parser.add_argument('-c', '--crd', help='coordinate filename')
    parser.add_argument('--browser', help='web browser')
    parser.add_argument('-j', '--jexe', default=default_jexe, help='jupyter path')
    parser.add_argument('--notebook-name', default='tmpnb_ngl.ipynb', help='notebook name')
    parser.add_argument('--port', type=int, help='port number')
    parser.add_argument('--remote', action='store_true', help='create remote notebook')
    parser.add_argument('--clean', action='store_true', help='delete temp file after closing notebook')
    parser.add_argument('--disable-autorun', action='store_true', help='do not run 1st cell right after openning notebook')
    parser.add_argument('--test', action='store_true', help='test')
    args = parser.parse_args(cmd_arg)

    command = args.command

    crd = args.traj if args.traj is not None else args.crd
    if crd is None:
        crd = command

    browser = '--browser ' + args.browser if args.browser else ''

    create_new_nb = False

    if command is not None and command.endswith('.ipynb'):
        notebook_name = command
    else:
        notebook_name = args.notebook_name
        if not PY3:
            kernelspec = notebook_dict['metadata']['kernelspec']
            kernelspec['display_name'] = 'Python 2'
            kernelspec['name'] = 'python2'
            notebook_dict['metadata']['kernelspec'] = kernelspec

            notebook_dict['metadata']['language_info']['codemirror_mode']['version'] = pyv_short_string
            notebook_dict['metadata']['version'] = pyv_full_string
        if command is None:
            # create a notebook and import nglview
            notebook_dict['cells'][0]['source'] = simple_source
            nb_json = json.dumps(notebook_dict)
            nb_json = nb_json.replace('"null"', 'null')
        elif command.endswith('.py'):
            # a Python script
            pycontent = open(command).read().strip()
            notebook_dict['cells'][0]['source'] = pycontent
            nb_json = json.dumps(notebook_dict)
        elif command == 'demo':
            # running demo
            notebook_dict['cells'][0]['source'] = demo_source
            nb_json = json.dumps(notebook_dict)
            nb_json = nb_json.replace('"null"', 'null').replace('test.nc', crd).replace('prmtop', command)
        elif _is_density_data(command):
            # check if density data
            notebook_dict['cells'][0]['source'] = density_source.replace('filename',
                    command)
            nb_json = json.dumps(notebook_dict)
        else:
            nb_json = json.dumps(notebook_dict)
            nb_json = nb_json.replace('"null"', 'null').replace('test.nc',
                    crd).replace('prmtop', command)
            assert os.path.exists(command), '{} does not exists'.format(command)

        nb_json = nb_json.replace('"null"', 'null')

        with open(notebook_name, 'w') as fh:
            fh.write(nb_json)
            create_new_nb = True
    
    
    dirname = os.path.dirname(os.path.abspath(notebook_name))
    if not args.remote:
        cm = '{jupyter} notebook {notebook_name} {browser}'.format(jupyter=args.jexe,
                                                                   notebook_name=notebook_name,
                                                                   browser=browser)
    else:
        port = get_remote_port(args.port, notebook_name)
        cm = '{jupyter} notebook --no-browser --port {port} ' \
              '--notebook-dir {dirname}'.format(jupyter=args.jexe,
                                                port=port,
                                                dirname=dirname)
        print('NOTE: make sure to open {0} in your local machine\n'.format(notebook_name))

    if args.disable_autorun or command is None:
        try:
            disable_extension(jupyter=args.jexe)
        except CalledProcessError:
            pass
    else:
        if not args.test:
            install_nbextension(jupyter=args.jexe)

    try:
        if args.test:
            # for testing cli
            pass
        else:
            subprocess.check_call(cm.split())
    except KeyboardInterrupt:
        if args.clean and create_new_nb:
            print("deleting {}".format(notebook_name))
            os.remove(notebook_name)
        if not args.disable_autorun:
            disable_extension(jupyter=args.jexe)

if __name__ == '__main__':
    main(cmd_arg=sys.argv[1:])



import argparse
from os.path import join
from notebook import nbextensions
from glob import glob


def install_nglview_js_widgets(user=True, symlink=False, overwrite=True, debug=False, **kwargs):
    """Install nglview-js-widgets nbextension.

    Parameters
    ----------
    user: bool, default True
        Install for current user instead of system-wide.
    symlink: bool, default False
        Symlink instead of copy (for development).
    overwrite: bool, default True
        Overwrite previously-installed files for this extension
    **kwargs: keyword arguments
        Other keyword arguments passed to the install_nbextension command
    """
    nglivew_js_dirs = nbextensions.install_nbextension_python('nglview',
            user=user, symlink=symlink, overwrite=overwrite, **kwargs)
    if debug:
        print(nglivew_js_dirs)
        print([glob(join(my_dir, '*')) for my_dir in nglivew_js_dirs])

def enable_nglview_js_widgets(user=True):
    nbextensions.enable_nbextension_python('nglview', user=user)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="nglview-js-widgets")
    parser.add_argument("-u", "--user",
                        help="Install as current user instead of system-wide",
                        action="store_true")
    parser.add_argument("-s", "--symlink",
                        help="Symlink instead of copying files",
                        action="store_true")
    parser.add_argument("-f", "--force",
                        help="Overwrite any previously-installed files for this extension",
                        action="store_true")
    parser.add_argument("-d", "--debug",
                        help="print nglivew-js-widgets",
                        action="store_true")
    args = parser.parse_args()
    install_nglview_js_widgets(user=args.user,
            symlink=args.symlink,
            overwrite=args.force,
            debug=args.debug)
    enable_nglview_js_widgets()



COLOR_SCHEMES = [
    " ",
    "picking",
    "random",
    "uniform",
    "atomindex",
    "residueindex",
    "chainindex",
    "modelindex",
    "sstruc",
    "element",
    "resname",
    "bfactor",
    "hydrophobicity",
    "value",
    "volume",
    "occupancy"]
